"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("util");
const ganache_core_1 = __importDefault(require("ganache-core"));
const ethers_1 = require("ethers");
const web3_1 = __importDefault(require("web3"));
const msgbus_1 = require("@securerpc/msgbus");
const lib_1 = require("../lib");
const MockDashboard_1 = __importDefault(require("./MockDashboard"));
jest.setTimeout(200000);
describe('DashboardProvider', () => {
    const ganachePort = 8545;
    const dashboardPort = 8546;
    let dashboardProvider;
    let mockDashboard;
    let messageBusPorts;
    let ganacheServer;
    beforeAll((done) => {
        ganacheServer = ganache_core_1.default.server();
        ganacheServer.listen(ganachePort, done);
    });
    afterAll((done) => {
        ganacheServer === null || ganacheServer === void 0 ? void 0 : ganacheServer.close(done);
    });
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        mockDashboard = new MockDashboard_1.default(ganacheServer.provider);
        dashboardProvider = new lib_1.DashboardProvider({
            dashboardPort,
            autoOpen: false,
        });
        messageBusPorts = yield (0, msgbus_1.getMessageBusPorts)(dashboardPort);
    }));
    afterEach(() => {
        mockDashboard.disconnect();
    });
    describe('Direct usage', () => {
        it('should retrieve unlocked accounts', () => __awaiter(void 0, void 0, void 0, function* () {
            const send = (0, util_1.promisify)(dashboardProvider.send.bind(dashboardProvider));
            // First connect the dashboard
            yield mockDashboard.connect(messageBusPorts.subscribePort);
            // Then send the request
            const response = yield send({
                jsonrpc: '2.0',
                method: 'eth_accounts',
                params: [],
                id: 1,
            });
            expect(response).toHaveProperty('result');
            expect(response === null || response === void 0 ? void 0 : response.result[0]).toBeDefined();
        }));
        it('should retrieve unlocked accounts if request gets sent before dashboard connects', () => __awaiter(void 0, void 0, void 0, function* () {
            const send = (0, util_1.promisify)(dashboardProvider.send.bind(dashboardProvider));
            // First send the request
            const request = send({
                jsonrpc: '2.0',
                method: 'eth_accounts',
                params: [],
                id: 1,
            });
            // Then connect the dashboard
            yield mockDashboard.connect(messageBusPorts.subscribePort);
            // Then await the response
            const response = yield request;
            expect(response).toHaveProperty('result');
            expect(response === null || response === void 0 ? void 0 : response.result[0]).toBeDefined();
        }));
        it('should send ETH', () => __awaiter(void 0, void 0, void 0, function* () {
            var _a, _b;
            const send = (0, util_1.promisify)(dashboardProvider.send.bind(dashboardProvider));
            yield mockDashboard.connect(messageBusPorts.subscribePort);
            const { result: accounts } = (_a = (yield send({
                jsonrpc: '2.0',
                method: 'eth_accounts',
                params: [],
                id: 1,
            }))) !== null && _a !== void 0 ? _a : {};
            const transaction = {
                from: accounts[0],
                to: accounts[1],
                value: '16345785D8A0000', // 0.1 ETH
            };
            const { result: hash } = (_b = (yield send({
                jsonrpc: '2.0',
                method: 'eth_sendTransaction',
                params: [transaction],
                id: 1,
            }))) !== null && _b !== void 0 ? _b : {};
            expect(hash).toBeDefined();
        }));
    });
    describe('Usage with Ethers.js', () => {
        let ethersProvider;
        beforeEach(() => {
            ethersProvider = new ethers_1.providers.Web3Provider(dashboardProvider);
        });
        afterEach(() => __awaiter(void 0, void 0, void 0, function* () {
            // Ethers sends a request to get the chainId on connection, so we await it
            // to make sure that the provider is finished with that request as well
            yield ethersProvider.ready;
        }));
        it('should retrieve unlocked accounts', () => __awaiter(void 0, void 0, void 0, function* () {
            // First connect the dashboard
            yield mockDashboard.connect(messageBusPorts.subscribePort);
            // Then send the request
            const accounts = yield ethersProvider.listAccounts();
            expect(accounts[0]).toBeDefined();
        }));
        it('should retrieve unlocked accounts if request gets sent before dashboard connects', () => __awaiter(void 0, void 0, void 0, function* () {
            // First send the request
            const request = ethersProvider.listAccounts();
            // Then connect the dashboard
            yield mockDashboard.connect(messageBusPorts.subscribePort);
            // Then await the response
            const accounts = yield request;
            expect(accounts[0]).toBeDefined();
        }));
        it('should send ETH', () => __awaiter(void 0, void 0, void 0, function* () {
            yield mockDashboard.connect(messageBusPorts.subscribePort);
            const accounts = yield ethersProvider.listAccounts();
            const signer = ethersProvider.getSigner();
            const response = yield signer.sendTransaction({
                from: accounts[0],
                to: accounts[1],
                value: ethers_1.utils.parseEther('0.1'),
            });
            expect(response).toHaveProperty('hash');
            expect(response.hash).toBeDefined();
        }));
        it('should send ETH even when dashboard loses connection as long as keepAlive is true', () => __awaiter(void 0, void 0, void 0, function* () {
            dashboardProvider.keepAlive = true;
            yield mockDashboard.connect(messageBusPorts.subscribePort);
            const accounts = yield ethersProvider.listAccounts();
            // Disconnect + reconnect the dashboard between DashboardProvider requests
            mockDashboard.disconnect();
            yield mockDashboard.connect(messageBusPorts.subscribePort);
            const signer = ethersProvider.getSigner();
            const response = yield signer.sendTransaction({
                from: accounts[0],
                to: accounts[1],
                value: ethers_1.utils.parseEther('0.1'),
            });
            // Manually terminate the provider
            dashboardProvider.terminate();
            expect(response).toHaveProperty('hash');
            expect(response.hash).toBeDefined();
        }));
    });
    describe('Usage with Web3.js', () => {
        let web3;
        beforeEach(() => {
            web3 = new web3_1.default(dashboardProvider);
        });
        it('should retrieve unlocked accounts', () => __awaiter(void 0, void 0, void 0, function* () {
            // First connect the dashboard
            yield mockDashboard.connect(messageBusPorts.subscribePort);
            // Then send the request
            const accounts = yield web3.eth.getAccounts();
            expect(accounts[0]).toBeDefined();
        }));
        it('should retrieve unlocked accounts if request gets sent before dashboard connects', () => __awaiter(void 0, void 0, void 0, function* () {
            // First send the request
            const request = web3.eth.getAccounts();
            // Then connect the dashboard
            yield mockDashboard.connect(messageBusPorts.subscribePort);
            // Then await the response
            const accounts = yield request;
            expect(accounts[0]).toBeDefined();
        }));
        it('should send ETH', () => __awaiter(void 0, void 0, void 0, function* () {
            yield mockDashboard.connect(messageBusPorts.subscribePort);
            const accounts = yield web3.eth.getAccounts();
            const response = yield web3.eth.sendTransaction({
                from: accounts[0],
                to: accounts[1],
                value: web3.utils.toWei('0.1'),
            });
            expect(response).toHaveProperty('transactionHash');
            expect(response.transactionHash).toBeDefined();
        }));
        it('should send ETH even when dashboard loses connection as long as keepAlive is true', () => __awaiter(void 0, void 0, void 0, function* () {
            dashboardProvider.keepAlive = true;
            yield mockDashboard.connect(messageBusPorts.subscribePort);
            const accounts = yield web3.eth.getAccounts();
            // Disconnect + reconnect the dashboard between DashboardProvider requests
            mockDashboard.disconnect();
            yield mockDashboard.connect(messageBusPorts.subscribePort);
            const response = yield web3.eth.sendTransaction({
                from: accounts[0],
                to: accounts[1],
                value: web3.utils.toWei('0.1'),
            });
            // Manually terminate the provider
            dashboardProvider.terminate();
            expect(response).toHaveProperty('transactionHash');
            expect(response.transactionHash).toBeDefined();
        }));
    });
});
//# sourceMappingURL=DashboardProvider.test.js.map